package model;

import util.Position;
import java.util.*;

public class NeighborManager {
    private final Map<Position, List<Position>> neighbors;

    public NeighborManager(int rowCount, int columnCount) {
        neighbors = new HashMap<>();
        for (int row = 0; row < rowCount; row++) {
            for (int col = 0; col < columnCount; col++) {
                Position position = new Position(row, col);
                neighbors.put(position, calculateAdjacentPositions(row, col, rowCount, columnCount));
            }
        }
    }

    /**
     * Calculates the list of adjacent positions for a given cell in the grid.
     *
     * @param row the row of the position
     * @param col the column of the position
     * @param rowCount the total number of rows in the grid
     * @param columnCount the total number of columns in the grid
     * @return a list of adjacent positions for the given cell
     */
    private List<Position> calculateAdjacentPositions(int row, int col, int rowCount, int columnCount) {
        List<Position> adjacentPositions = new ArrayList<>();
        if (row > 0) adjacentPositions.add(new Position(row - 1, col));
        if (col > 0) adjacentPositions.add(new Position(row, col - 1));
        if (row < rowCount - 1) adjacentPositions.add(new Position(row + 1, col));
        if (col < columnCount - 1) adjacentPositions.add(new Position(row, col + 1));
        return adjacentPositions;
    }

    /**
     * Returns an unmodifiable view of the neighbors map to ensure data encapsulation.
     *
     * @return an unmodifiable map of positions and their respective neighbors
     */
    public Map<Position, List<Position>> getNeighbors() {
        return Collections.unmodifiableMap(neighbors);
    }

    /**
     * Returns the list of neighbors for a specific position.
     *
     * @param position the position for which neighbors are requested
     * @return a list of adjacent positions for the given position
     */
    public List<Position> getAdjacentPositions(Position position) {
        return neighbors.getOrDefault(position, Collections.emptyList());
    }
}
